@@ -4,9 +4,9 @@
  * GChemPaint
  * main.cc 
  *
- * Copyright (C) 2001-2004
+ * Copyright (C) 2001-2005
  *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -65,6 +65,10 @@ static GtkMenu *windowsmenu;
 static GtkWidget* bar;	//GnomeAppBar
 static map<string, string> SupportedPixbufFormats;
 
+#ifndef GTK_IS_26
+GdkPixbuf *icon;
+#endif
+
 void on_change_element(int Z)
 {
 	CurElt = Z;
@@ -330,6 +334,7 @@ static bool do_save_as_image(GtkWidget* widget, GtkFileSelection *fsel)
 	if (!filename || !strlen(filename) || filename[strlen(filename) - 1] == '/') {
 		GtkWidget* message = gtk_message_dialog_new (GTK_WINDOW (fsel), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
 															_("Please enter a file name,\nnot a directory"));
+		SET_ICON(message);
 		gtk_dialog_run (GTK_DIALOG (message));
 		gtk_widget_destroy (message);
 		return true;
@@ -342,6 +347,7 @@ static bool do_save_as_image(GtkWidget* widget, GtkFileSelection *fsel)
 	{
 		gchar * message = g_strdup_printf(_("File %s\nexists, overwrite?"), filename);
 		GtkDialog* Box = GTK_DIALOG(gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
+		SET_ICON(message);
 		result = gtk_dialog_run(Box);
 		gtk_widget_destroy(GTK_WIDGET(Box));
 		g_free(message);
@@ -365,6 +371,7 @@ static bool do_save_as_image(GtkWidget* widget, GtkFileSelection *fsel)
 		} else {
 			GtkWidget* message = gtk_message_dialog_new (GTK_WINDOW (fsel), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, 
 																_("Please, install which and ps2eps!"));
+			SET_ICON(message);
 			gtk_dialog_run (GTK_DIALOG (message));
 			gtk_widget_destroy (message);
 		}
@@ -379,6 +386,7 @@ static bool do_save_as_image(GtkWidget* widget, GtkFileSelection *fsel)
 	else {
 		GtkWidget* message = gtk_message_dialog_new (GTK_WINDOW (fsel), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
 															_("Sorry, format not supported!"));
+		SET_ICON(message);
 		gtk_dialog_run (GTK_DIALOG (message));
 		gtk_widget_destroy (message);
 	}
@@ -406,11 +414,12 @@ Costantino Ceoldo: Italian\n\
 Ben Luo: Chinese\n\
 Michał Sałaban: Polish"
 			);
-	GdkPixbuf* pixbuf = gdk_pixbuf_new_from_file ( DATADIR"/pixmaps/gchempaint_logo.png", NULL);
+	GdkPixbuf* pixbuf = gdk_pixbuf_new_from_file ( DATADIR"/"PACKAGE"/pixmaps/gchempaint_logo.png", NULL);
 	GtkWidget* about = gnome_about_new(_("GChemPaint"), VERSION,
 										_("(C) 2001-2004 by Jean Bréfort"),
 										_("GChemPaint is a 2D chemical structures editor for Gnome"),
 										(const gchar**) authors, (const gchar**) documentors, translators, pixbuf);
+	SET_ICON(about);
 	if (pixbuf != NULL)
 		g_object_unref (pixbuf);
 	gtk_widget_show_all(about);
@@ -583,6 +592,14 @@ int main(int argc, char *argv[])
 		l = l->next;
 	}
 	g_slist_free (formats);
+#ifndef GTK_IS_26
+	GError *error = NULL;
+#ifdef GNOME_OLD_VER
+	icon = gdk_pixbuf_new_from_file (DATADIR"/pixmaps/gchempaint.png", &error);
+#else
+	icon = gdk_pixbuf_new_from_file (DATADIR"/icons/hicolor/48x48/apps/gchempaint.png", &error);
+#endif
+#endif
 
 	if (bonobo_server_flag)
 	{
@@ -594,6 +611,7 @@ int main(int argc, char *argv[])
 		app = GNOME_APP(gnome_app_new(_("GChemPaint"), _("GChemPaint")));
 		gnome_app_create_menus(app, main_menu);
 		gnome_app_create_toolbar(app, tool_bar);
+		SET_ICON(app);
 		windowsmenu = GTK_MENU(windows_menu[0].widget->parent);
 		bar = gnome_appbar_new(true, true, GNOME_PREFERENCES_ALWAYS);
 		gnome_appbar_set_default(GNOME_APPBAR(bar), _("Ready."));
@@ -622,6 +640,10 @@ int main(int argc, char *argv[])
 		SubscriptBtn = glade_xml_get_widget(xml, "subscript");
 		SuperscriptBtn = glade_xml_get_widget(xml, "superscript");
 		FontBtn = glade_xml_get_widget(xml, "font");
+#ifdef GTK_IS_RECENT
+		gtk_toolbar_set_show_arrow(GTK_TOOLBAR (TextToolBar), false);
+#endif
+		gtk_widget_show_all (TextToolBar);
 		gnome_app_add_toolbar(app, GTK_TOOLBAR(widget), "texttools", BONOBO_DOCK_ITEM_BEH_NORMAL, BONOBO_DOCK_TOP, 2, 0, 0);
 		gtk_widget_set_sensitive(widget, false);
 		g_object_unref(G_OBJECT(widget));
@@ -646,6 +668,10 @@ int main(int argc, char *argv[])
 
 		gtk_main();
 	}
+
+#ifndef GTK_IS_26
+	g_object_unref (icon);
+#endif
 		
 	return 0;
 }
